/*==========================================================================;
 *
 *  AUTHOR:     Subash, Agila
 *
 *  FILE NAME:  symtable.h
 *
 *  CONTENTS:  Definition of the symbol table.
 *              class SymbolTable, union STVal
 *
 *  DATE:      March 19, 2004
 ***************************************************************************/

 /*
Specification file for the symTable(Symbol Table)
*/
#ifndef SymTableh
#define SymTableh


#include "memory.h"

	// Default values of some fields
#define ST_FREE         -1
#define TYPE_NONE        0

	// Initial sizes of the hash table and node table:
#define MAX_NT_ELEMENTS                 32
#define MAX_HT_ELEMENTS                 61
    // This number governs reallocation of the hash table.
    // When numberOfInsertedElements/sizeOfHashTable >= HASH_REALLOCATE_RATIO,
    //  the reallocation occurrs.
#define HASH_REALLOCATE_RATIO    (float)1


//initial size of spell table
#define INITIAL_SPELL_SIZE              100
//  Maximal size of the piece for spell table
#define MAX_SPELL_SIZE                  10000
// Default growth factor for spell table
#define SPELL_GROWTH_FACTOR             2.0
// Size of hash table will be multiplied by this number during reallocation
#define HASH_GROWTH_FACTOR              2.0
// Size of node table will be multiplied by this number during reallocation
#define NODE_GROWTH_FACTOR              2.0




union STVal;
struct STNodeType;  //forward declaration.  See implementation file for details

// Union defining the value of an entry in the symbol table.
union STVal
{
int number;   // Integer value of the entry
              //   (will be used for integer constants only )
void *ptr;    // Pointer value to some node(for the rest)
};


// MAIN CLASS:
class SymbolTable
{
public:
	// Constructor & destructor:
	SymbolTable();
	~SymbolTable();

	// Insert or find a name in the symbol table:
	//  Name -> SymbolNumber
	int insertName(char*);

	char * getName(int num); // SymbolNumber -> Name

	int getType(int num);	// SymbolNumber -> entryType
	void setType(int num,int type1); // Set the type of the entry

	     // Get the pointer to the value of the entry. Can be used for both
	     //  reading and writing the value:
	STVal * getVal(int num);

	// We need this to remove all entries from the symbol table
	//  so that it can be used again.
	void clear();

	void print(); // For purpose of debugging the remaining program.

		// Return the range of the symbol numbers that are in use
		//  (symbol numbers are assigned consequently):
	void getNumRange(int *min, int *max);

private:

	MemAlloc mem;               // Spelling table
	struct STNodeType *STNode;  //node Table
	int* hashTable;        //field next in the STNode will contain the
	                       // ref to the next node with the same hash value
	int cntNTEntry;        //Node Table count of entries
	int hashTableSize;     //initial hash table size
	int nodeTableSize;     //initial node Table size

	void HTRealloc();      // Reallocate the hash table

	void NTRealloc();      // Reallocate the node table

	int calHashValue(char *);// Hash function.
	                    //calculates hash value for the given character

};
#endif
